/*
ESUP-portail is a french academic project developed under the GPL (General Public License) augmented according to the following :
A binary or source file developped by ESUP-portail can be used or compiled with products under Apache license.
The official english text of the GPL can be found here : http://www.gnu.org/licenses/gpl.html .
A non official french translation can be found here : http://www.linux-France.org/article/these/gpl.html .
The different kinds of licenses governing the products developed by the Apache foundation can be found here : http://www.apache.org/licenses .
It follows that you can as well as use download contents as well modify and redistribute them provided you respect the GPL terms.
Downloading and using such contents do not provide any guaranty.
Be sure that you have well understood the terms of the license before using the contents it covers.
The ESUP-portail distribution includes the following distributions :
    * UPortal :
      software developed by JA-SIG (Java Architecture - Special Interest Group)
      You can find the license page here : http://mis105.udel.edu/ja-sig/uportal/license.html
    * CAS :
      SSO solution developed by Yale University
      You can find the project page here : http://www.yale.edu/tp/auth
    * Cocoon :
      XML framework distributed by the Apache foundation under Apache license;
      Please find the full text here : http://cocoon.apache.org/2.1/license.html
    * Mod_dav:
      A DAV module for Apache web server
      You can find the project page here : http://www.webdav.org/mod_dav
    * IMP :
      webmail from Horde application framework
      You can find the project page here : http://www.horde.org
    * . To be completed
*/
 
package org.esupportail.portal.channels.CPref;   
 
 
import java.util.Hashtable;
import org.esupportail.portal.channels.CPref.config.Config;
import org.esupportail.portal.channels.CPref.config.LdapAttribut;
import org.esupportail.portal.channels.CPref.formats.Format;
import org.esupportail.portal.utils.channels.FrameWorkException;
import org.esupportail.portal.utils.channels.MainChannel;
import org.esupportail.portal.utils.channels.plugins.Message;
import org.esupportail.portal.utils.channels.plugins.MessageBean;
import org.jasig.portal.ChannelRuntimeData;
import org.jasig.portal.PortalException;
import org.jasig.portal.security.IPerson;
import org.jasig.portal.services.LogService;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.SearchResult;
import org.esupportail.portal.utils.channels.plugins.RestrictedAction;
import org.esupportail.portal.channels.CPref.LDAP.Exceptions.LDAPException;


public abstract class UpdateAction  extends SecureAction{ 
	
	/**
	 * @param mainChannel
	 */
	public UpdateAction(MainChannel mainChannel) {
		super(mainChannel);
		password = null;
	}

	private String actionVisu = "";
	private String actionModif = "";
	private String actionValid = "";
	private String[] ldapFieldsToModify = new String[0];
	private String[] ldapFieldsToDisplay = new String[0];
	
/**********************************************/
/************FONCTIONS ABSTRAITES**************/
	/**
	 * Valeurs cls de l'action
	 */	
	public abstract boolean prepare() throws FrameWorkException;

	/**
	 * Spcification des champs LDAP  utiliser en lecture
	 * Ceux spcifis en write sont pris en compte aussi dans les read
	 */	
	public abstract String[] prepareListFieldRead() throws FrameWorkException;	
	
	/**
	 * Spcification des champs LDAP  utiliser en criture
	 */	
	public abstract String[] prepareListFieldWrite() throws FrameWorkException;	
	
/**********************************************/
/******FONCTIONS IMPLEMENTEES PAR DEFAUT*******/
	/**
	 * Mthode effectue une fois authentifi
	 * @return Boolean : 
	 * 	* Boolean.TRUE: validation effectue avec succs
	 * 	* Boolean.FALSE: redirection sur le plugin message
	 */
	public Boolean identified() throws FrameWorkException{
		try {
			return visualisation();
		} catch (FrameWorkException e) {
			try {
				Message.message(mainChannel,runtimeData,new MessageBean(e.getDisplayMessage(),Message.ERROR),"default",new Hashtable());
			} catch (PortalException e1) {
				LogService.log(LogService.ERROR,"PortalException : "+e1);
			}
			return Boolean.FALSE;
		} catch (PortalException e) {
			try {
				Message.message(mainChannel,runtimeData,new MessageBean(e.getMessage(),Message.ERROR),"default",new Hashtable());
			} catch (PortalException e1) {
				LogService.log(LogService.ERROR,"PortalException : "+e1);
			}
			return Boolean.FALSE;
		}
	}	

	/**
	 * Mthode permettant d'enregistrer les donnes
	 */
	public Boolean validation() throws FrameWorkException, PortalException {
		
		/*Ouverture de la connextion LDAP*/ 
		this.openLDAP(true);   
		
		/*Positionnement des valeurs  enregistrer*/
		String[] attribsValue = new String[ldapFieldsToModify.length];	
		for (int i=0; i<ldapFieldsToModify.length;i++){
			String paramName = getModifiedFileds()[i];
			if (runtimeData.getParameter(paramName)!=null){
				attribsValue[i] = runtimeData.getParameter(paramName);	
			}
			else{
				attribsValue[i] = "";
			}
		}  
		LDAPconnexion.setAttributesValues(attribsValue);
		
		LDAPconnexion.setAttributes(ldapFieldsToModify);
		
		/*criture*/
		try {
			LDAPconnexion.write(dn);
		} catch (LDAPException e) {
			try {
				Message.message(mainChannel,runtimeData,new MessageBean(e.getDisplayMessage()),"default",new Hashtable());
			} catch (PortalException e1) {
				LogService.log(LogService.ERROR,"PortalException : "+e1);
			}
			return Boolean.FALSE;
		}
		
		/*Fermeture de la connextion LDAP*/ 
		this.closeLDAP();
		
		return Boolean.TRUE;
	}

/**********************************************/
/************FONCTION DU MAG*******************/	
	public Boolean init(ChannelRuntimeData rd) throws PortalException,FrameWorkException {
		super.init(rd);
		
		/* 
		 * Si l'utilisateur clique sur le bouton "Cancel" alors
		 * le MAG est redirig sur l'action par dfault
		 */
		String cancel = runtimeData.getParameter("Cancel");
		if (cancel!=null){
			mainChannel.redirect(rd, "default");
			return Boolean.FALSE;
		}
		 
		LogService.log(LogService.INFO,"InfoPers:init() : "+(String)mainChannel.getStaticData().getPerson().getAttribute(IPerson.USERNAME)+" essaye de se connecter  l'action "+mainChannel.getCurrentAction().getName());
		
		/*
		 * Vrification de l'autorisation d'accs
		 */
		if (!RestrictedAction.filterAccess(mainChannel,mainChannel.getCurrentAction().getName(),Config.getInstance().getGroupmapping())){
		    LogService.log(LogService.ERROR,"InfoPers:init() : "+(String)mainChannel.getStaticData().getPerson().getAttribute(IPerson.USERNAME)+" n'est pas autoris  se connecter  l'action "+mainChannel.getCurrentAction().getName());
			Message.message(mainChannel, runtimeData, new MessageBean("Vous n'tes pas autoris  accder  cette page."),"default",new Hashtable());
			return Boolean.FALSE;
		}
		
		
		
		
		/*
		 * Appel de la mthode prpare
		 */
		if (!prepare())
			return Boolean.FALSE;
		setDisplayFileds(prepareListFieldRead());
		setModifiedFileds(prepareListFieldWrite());
		
		/*
		 * Si l'action de validation est appelle directement sans passer par
		 * le formulaire de modification
		 * 		 
		 */
		if ((mainChannel.getCurrentAction().getName().equals(actionValid))
				&& (!mainChannel.getPreviousAction().getName().equals(actionModif))
				){
			setForbidden(true);
			setIdentidied(false);
			return Boolean.TRUE;
		}
		
		

		
		/*
		 * Enregistrement de la saisie
		 *  
		 */
		if (mainChannel.getCurrentAction().getName().equals(actionValid)){
			try {
				
				boolean validForm = true;
				
				try {
					validForm = verifAttribLDAP(ldapFieldsToModify); 
				} catch (FrameWorkException e1) {
					try {
						Message.message(mainChannel,runtimeData,new MessageBean(e1.getDisplayMessage()),"default",new Hashtable());
					} catch (PortalException e) {
						LogService.log(LogService.ERROR,"InfoPers:init() : PortalException "+e);
					}
					return Boolean.FALSE;
				}
				
				if (!validForm){
					try {
						mainChannel.redirect(runtimeData,mainChannel.getPreviousAction().getName());
					} catch (PortalException e) {
						LogService.log(LogService.ERROR,"InfoPers:init() : PortalException "+e);
					}
					return Boolean.FALSE;
				}
				
				
				Boolean res = validation();
				setNonSecure(true);
				deletePassword();
				return res;
			} catch (PortalException e) {
				throw new FrameWorkException("Erreur dans la validation");
			}
			
		}

		
		/*
		 * Si l'action de modification est appelle apres la validation c'est
		 * qu'une erreur de saisie s'est glisse dans la saisie
		 * Il ne faut donc pas effacer le mot de passe
		 * 		
		 */
		if ( (mainChannel.getCurrentAction().getName().equals(actionModif))
				&&(mainChannel.getPreviousAction().getName().equals(actionValid))
			){
			return Boolean.TRUE;
		}		
		
		deletePassword();
		setNonSecure(false);
		setForbidden(false);
		if (mainChannel.getCurrentAction().getName().equals(actionVisu)){
			setNonSecure(true);
		}
		
		return Boolean.TRUE;
	}
	
	
	
/**********************************************/
/************FONCTION D'ACTION*****************/
	/**
	 * fonction a effectuer sans authentification
	 */	
	public Boolean visualisation() throws FrameWorkException,PortalException{
		
		String[] listAttributesLdap = new String[ldapFieldsToDisplay.length+ldapFieldsToModify.length];
		
		/*Rcupration des champs en lecture*/
		for (int i=0;i<ldapFieldsToDisplay.length;i++){
			LdapAttribut attrib = (LdapAttribut)Config.getInstance().getAttribLdap(ldapFieldsToDisplay[i]);
			if (attrib==null){
				LogService.log(
						LogService.ERROR,
						"UpdateAction:visualisation() : Le champ '"+ldapFieldsToDisplay[i]+"' est utilis mais pas dclar dans la config.");
				throw new FrameWorkException("Un des attributs  utiliser n'est pas dclar.");
			}
			listAttributesLdap[i] = attrib.getLdapName();
		}
		/*Rcupration des champs en criture*/
		for (int i=ldapFieldsToDisplay.length;i<ldapFieldsToDisplay.length+ldapFieldsToModify.length;i++){
			LdapAttribut attrib = (LdapAttribut)Config.getInstance().getAttribLdap(ldapFieldsToModify[i-ldapFieldsToDisplay.length]);
			if (attrib==null){
				LogService.log(
						LogService.ERROR,
						"UpdateAction:visualisation() : Le champ '"+ldapFieldsToDisplay[i]+"' est utilis mais pas dclar dans la config.");
				throw new FrameWorkException("Un des attributs  utiliser n'est pas dclar.");
			}
			listAttributesLdap[i] = attrib.getLdapName();
		}
		
		/*Ouverture d'une connexion*/
		this.openLDAP();
		
		/*Positionnement des attributs  rcuprer*/
		LDAPconnexion.setAttributes(listAttributesLdap);
		
		NamingEnumeration answer = null; 
		try {
			answer = LDAPconnexion.search();
		} catch (LDAPException e1) {
			LogService.log(LogService.ERROR,"UpdateAction:visualisation() : LDAPException : "+e1);
			throw new FrameWorkException(e1.getMessage());
			
		}
		
		
		StringBuffer xmlBuffer = new StringBuffer();  
		try {				
			/*Parcours des rsultats obtenus et cration de la chaine XML*/
			while (answer.hasMoreElements()) {
				int errorForm = 0;
				SearchResult sr;
				sr = (SearchResult) answer.next();					
				xmlBuffer.append("<INFOS ")  ;
				/*Parcours des attributs des rsultats*/
				for (int i=0;i<listAttributesLdap.length;i++){
					
					String nomAttrib = listAttributesLdap[i];
					if (runtimeData.getParameter(nomAttrib)!=null){
						/*Traitement des erreurs de saisie*/
						xmlBuffer.append(nomAttrib+"=\""+(String)runtimeData.getParameter(nomAttrib)+"\" ");
						if (runtimeData.getParameter("ERROR_"+nomAttrib)!=null){
							xmlBuffer.append("error_"+nomAttrib+"=\"1\" ");
							errorForm++;
						}
					}						
					else if (sr.getAttributes().get(listAttributesLdap[i])!=null){
						xmlBuffer.append(nomAttrib+"=\""+(String)sr.getAttributes().get(listAttributesLdap[i]).get(0)+"\" ");					
					}
				}
				
				this.getXSLParameter().put("error",Integer.toString(errorForm));
				
				xmlBuffer.append("/>"); 
				
			}
			this.xml = xmlBuffer.toString();
		} catch (NamingException e) {
			LogService.log(
					LogService.ERROR,
					"UpdateAction:visualisation() : NamingException "+e);
			return Boolean.FALSE;
		}
		finally{
			this.closeLDAP();
		}
		
		return Boolean.TRUE;
	}

/**********************************************/
/****FONCTIONS DE VERIFICATION DE SAISIE*******/	
	/**
	 * Valide la saisie des atributs en fonction du format dfini
	 * 
	 * @param listAttrib : liste des champs  vrifier 
	 * @return true s'il n'y a pas d'erreur de saisie
	 */
	public boolean verifAttribLDAP(String[] listAttrib) throws FrameWorkException{
		boolean valid = true;
		
		if ( (listAttrib==null) || (listAttrib.length==0) ){
			LogService.log(LogService.ERROR,"UpdateAction::verifAttribLDAP: FrameWorkException : validation d'un formulaire dont les champs ne sont pas spcifis.");
			throw new FrameWorkException("Vous n'avez prcis aucun champ  valider.");
		}
		
		for (int i=0; i<listAttrib.length;i++){
			LdapAttribut attrib = (LdapAttribut)Config.getInstance().getAttribLdap(listAttrib[i]);
			
			if (attrib==null){
				LogService.log(LogService.ERROR,"UpdateAction::verifAttribLDAP: FrameWorkException : tentative de modification de "+listAttrib[i]+" alors que ce champ est n'est pas dclar en config.");
				throw new FrameWorkException("Vous essayez de modifier un champ non dclarer en configuration.");
			}

			if (!attrib.getAccess().equals("write")){
				LogService.log(LogService.ERROR,"UpdateAction::verifAttribLDAP: FrameWorkException : tentative de modification de "+attrib.getLdapName()+" alors que ce champ est n'est pas en mode write.");
				throw new FrameWorkException("Vous essayez de modifier un champ en lecture seule.");
			}
			
			String group = (String)mainChannel.getStaticData().get("plug.esup.restrictedAction");
			if (runtimeData.getParameter(attrib.getLdapName())!=null){
				Class c;
				Format format;
				try {
					if (attrib.getFormat(group)!=null){
						c = Class.forName(attrib.getFormat(group));
						format = (Format)c.newInstance();
						String valueAttrib = runtimeData.getParameter(attrib.getLdapName());	
						/*Vrifie si le format spcifi en config est respect*/
						if (!format.check(valueAttrib)){
							this.getRuntimeData().setParameter("ERROR_"+attrib.getLdapName(),"1");
							valid = false;
							LogService.log(LogService.INFO,"UpdateAction::verifAttribLDAP: "+attrib.getLdapName()+" : format \""+attrib.getFormat(group)+"\" non valide");							
						}
						else{
							/*Transformation selon le format*/
							runtimeData.setParameter(attrib.getLdapName(),format.transform(valueAttrib));
						}
					}
					
				} catch (ClassNotFoundException e) {
					LogService.log(LogService.ERROR,"UpdateAction::verifAttribLDAP: ClassNotFoundException : "+attrib.getFormat(group));
					throw new FrameWorkException("Impossible de trouver le format : "+attrib.getFormat(group));
				} catch (InstantiationException e) {
					LogService.log(LogService.ERROR,"UpdateAction::verifAttribLDAP: InstantiationException : "+attrib.getFormat(group));
					throw new FrameWorkException("Impossible d'instancier le format : "+attrib.getFormat(group));
				} catch (IllegalAccessException e) { 
					LogService.log(LogService.ERROR,"UpdateAction::verifAttribLDAP: IllegalAccessException : "+attrib.getFormat(group));
					throw new FrameWorkException("IllegalAccessException : "+attrib.getFormat(group));
				}			
			}
		}
		
		return valid;
	}

		
/**********************************************/	
/*****SETTER DES ACTIONS MAG REFERENCEES*******/
	/**
	 * @param actionVisu Le nom de l'action MAG pour avoir un apercu des valeurs
	 */
	public void setActionVisu(String actionVisu) {
		this.actionVisu = actionVisu;
	}
	
	/**
	 * @param actionModif Le nom de l'action MAG pour acceder au formulaire de modification
	 */
	public void setActionModif(String actionModif) {
		this.actionModif = actionModif;
	}
	
	/**
	 * @param actionValid Le nom de l'action MAG pour valider la saisie
	 */
	public void setActionValid(String actionValid) {
		this.actionValid = actionValid;
	}

/************************************************/	
/*****GETTER ET SETTER DES CHAMPS UTILISES*******/
	/**
	 * Positionne les champs LDAP  modifier.
	 * Tableau de String contenant les nom logiques des attributs 
	 * LDAP utiliss
	 * 
	 * @param ldapField tableau de champs LDAP qui seront modifis sur cette action
	 */
	public void setModifiedFileds(String[] fields) {
		if (fields==null)fields = new String [0];
		ldapFieldsToModify = fields;
	}
	
	/**
	 * Permet d'obtenir la liste des noms logiques des champs en modification
	 * 
	 * @return String[] : liste des noms logiques des champs
	 */
	public String[] getModifiedFileds() {
		if (ldapFieldsToModify==null)return new String[0];
		return ldapFieldsToModify;
	}
		
	/**
	 * Positionne les champs LDAP  afficher.
	 * Tableau de String contenant les nom logiques des attributs 
	 * LDAP utiliss
	 * 
	 * @param ldapField tableau de champs LDAP qui seront affichs sur cette action
	 */
	private void setDisplayFileds(String[] fields) {
		if (fields==null)fields = new String [0];
		ldapFieldsToDisplay = fields;
	}
	/**
	 * Permet d'obtenir la liste des noms logiques des champs en lecture
	 * 
	 * @return String[] : liste des noms logiques des champs
	 */
	public String[] getDisplayFileds() {
		if (ldapFieldsToDisplay==null)return new String[0];
		return ldapFieldsToDisplay;
	}
	


}
